-- FUNCTION:    RS_Control.vhd
-- DESCRIPTION: RS Control circuit
-- AUTHOR:      Christos Zamantzas
-- CREATED:     13/02/2008
-- UPDATE:      10/08/2009
-- VERSION:  7

-- Changes in ver.6: Initial version to svn.
-- Changes in ver.7: changed library (arith -> numeric)
--                   beutification of code

-- TODO: 

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.all;
USE IEEE.numeric_std.all;
USE IEEE.STD_LOGIC_UNSIGNED.all;

ENTITY RS_Control IS
   
   PORT(
      flag_0x, 
      flag_1x, 
      flag_2x, 
      flag_3x        : IN  STD_LOGIC;
      data_0x, 
      data_1x, 
      data_2x, 
      data_3x        : IN  STD_LOGIC_VECTOR (127 DOWNTO 0);
      ARM_Capture    : IN  STD_LOGIC;
      Start_Capture  : IN  STD_LOGIC;
      Stop_Capture   : IN  STD_LOGIC;
      clk            : IN  STD_LOGIC;
      data_out       : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
      wr_req         : OUT STD_LOGIC
      );

END RS_Control;

ARCHITECTURE RTL OF RS_Control IS

SIGNAL RS_Control_int : STD_LOGIC_VECTOR(3 DOWNTO 0);
SIGNAL data_0x_int, 
       data_1x_int, 
       data_2x_int, 
       data_3x_int    : STD_LOGIC_VECTOR(127 DOWNTO 0);
SIGNAL flag_0x_int,
       flag_1x_int,
       flag_2x_int,
       flag_3x_int    : STD_LOGIC;
SIGNAL iARM_Capture   : STD_LOGIC;
SIGNAL iStart_Capture : STD_LOGIC;
SIGNAL iStop_Capture  : STD_LOGIC;
SIGNAL iwr_req        : STD_LOGIC;

SIGNAL ARM            : STD_LOGIC;
SIGNAL Start          : STD_LOGIC;
SIGNAL Stop           : STD_LOGIC;
SIGNAL Endofpacket    : STD_LOGIC;

BEGIN
PROCESS (clk)

VARIABLE WordCnt      :integer RANGE 0 TO 15;
 
BEGIN

   IF  rising_edge(clk) 
      THEN
      -- Register flags -- 
      iARM_Capture    <= ARM_Capture;
      iStart_Capture  <= Start_Capture;
      iStop_Capture   <= Stop_Capture;
      
      -- disregard re-arming requests when in capture mode
      IF iARM_Capture = '1' AND Start ='0' THEN
         ARM    <='1';
      END IF;
      -- start capturing data
      IF iStart_Capture = '1' AND ARM ='1' THEN
         Start    <='1';
         ARM    <='0';
         flag_0x_int <= '0';  
         flag_1x_int <= '0'; 
         flag_2x_int <= '0';
         flag_3x_int <= '0';
         iwr_req  <= '0';
      END IF;
      -- stop capturing data
      IF iStop_Capture = '1' AND Start ='1' THEN
         Start    <='0';
      END IF;
      
      -- Register New Values -- 
      IF  flag_0x = '1' 
         THEN
         data_0x_int <= data_0x;
         flag_0x_int <= '1';
      END IF;
      IF  flag_1x = '1' 
         THEN
         data_1x_int <= data_1x;
         flag_1x_int <= '1';
      END IF;
      IF  flag_2x = '1' 
         THEN
         data_2x_int <= data_2x;
         flag_2x_int <= '1';
      END IF;
      IF  flag_3x = '1' 
         THEN
         data_3x_int <= data_3x;
         flag_3x_int <= '1';
      END IF;
      
      -- initialise and start writing -- 
      IF    iwr_req  = '0' AND
         flag_0x_int = '1' AND 
         flag_1x_int = '1' AND 
         flag_2x_int = '1' AND 
         flag_3x_int = '1' AND
         Start = '1'
         THEN   
         WordCnt := 0; 
         iwr_req <= '1';
         Endofpacket <= '0';
      END IF;
      
      ---------------------------------------
      -- Output New Values by 32 bit words --
      ---------------------------------------
      -- First RS                          --
      IF Wordcnt = 0 AND iwr_req = '1' THEN
         data_out <= data_0x_int(31 DOWNTO 0);
         WordCnt := WordCnt + 1;
      ELSIF Wordcnt = 1 THEN
         data_out <= data_0x_int(63 DOWNTO 32);
         WordCnt := WordCnt + 1;
      ELSIF Wordcnt = 2 THEN
         data_out <= data_0x_int(95 DOWNTO 64);
         WordCnt := WordCnt + 1;
      ELSIF Wordcnt = 3 THEN
         data_out <= data_0x_int(127 DOWNTO 96);
         WordCnt := WordCnt + 1;
      -- Second RS                          --
      ELSIF Wordcnt = 4 THEN
         data_out <= data_1x_int(31 DOWNTO 0);
         WordCnt := WordCnt + 1;
      ELSIF Wordcnt = 5 THEN
         data_out <= data_1x_int(63 DOWNTO 32);
         WordCnt := WordCnt + 1;
      ELSIF Wordcnt = 6 THEN
         data_out <= data_1x_int(95 DOWNTO 64);
         WordCnt := WordCnt + 1;
      ELSIF Wordcnt = 7 THEN
         data_out <= data_1x_int(127 DOWNTO 96);
         WordCnt := WordCnt + 1;
      -- Third RS                          --
      ELSIF Wordcnt = 8 THEN
         data_out <= data_2x_int(31 DOWNTO 0);
         WordCnt := WordCnt + 1;
      ELSIF Wordcnt = 9 THEN
         data_out <= data_2x_int(63 DOWNTO 32);
         WordCnt := WordCnt + 1;
      ELSIF Wordcnt = 10 THEN
         data_out <= data_2x_int(95 DOWNTO 64);
         WordCnt := WordCnt + 1;
      ELSIF Wordcnt = 11 THEN
         data_out <= data_2x_int(127 DOWNTO 96);
         WordCnt := WordCnt + 1;
      -- Forth RS                          --
      ELSIF Wordcnt = 12 THEN
         data_out <= data_3x_int(31 DOWNTO 0);
         WordCnt := WordCnt + 1;
      ELSIF Wordcnt = 13 THEN
         data_out <= data_3x_int(63 DOWNTO 32);
         WordCnt := WordCnt + 1;
      ELSIF Wordcnt = 14 THEN
         data_out <= data_3x_int(95 DOWNTO 64);
         WordCnt := WordCnt + 1;
      ELSIF Wordcnt = 15 AND Endofpacket = '0' THEN
         data_out <= data_3x_int(127 DOWNTO 96);
         -- END of writting and reset control flags --
         Endofpacket <= '1';
         flag_0x_int <= '0';  
         flag_1x_int <= '0'; 
         flag_2x_int <= '0';
         flag_3x_int <= '0';
         iwr_req  <= '0';
      END IF;
      wr_req <= iwr_req;
    END IF;
    
 END PROCESS;

END RTL;
