-- FILE: OutputValidStatuses.vhd
-- DESCRIPTION: Chooses the correct Statuses to forward.
-- VERSION: 6
-- DATE: 2009/12/14

-- Changes in ver. 2: Initial conversion to vhdl. BusMux blocks have been removed
-- Changes in ver. 3: clocked function.
-- Changes in ver. 4: renamed signal validsig to validPacket
-- Changes in ver. 5: added an enableRead input signal; data should not be read when not needed.
-- Changes in ver. 6: added an one shot to the enableRead input signal; data should be read once.


LIBRARY ieee;
USE ieee.std_logic_1164.all; 

LIBRARY work;

ENTITY OutputValidStatuses IS 
   PORT
   (
      CID_ChA     :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      CID_ChB     :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      
      FID_ChA     :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      FID_ChB     :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      
      StatusA_ChA :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      StatusA_ChB :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      
      StatusB_ChA :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      StatusB_ChB :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      
      DAC_1_2_ChA :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      DAC_3_4_ChA :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      DAC_5_6_ChA :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      DAC_7_8_ChA :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      
      DAC_1_2_ChB :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      DAC_3_4_ChB :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      DAC_5_6_ChB :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      DAC_7_8_ChB :  IN  STD_LOGIC_VECTOR(15 DOWNTO 0);
      
      validPacket :  IN  STD_LOGIC;
      enableRead  :  IN  STD_LOGIC;
      clk         :  IN  STD_LOGIC;
      
      CID         :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
      FID         :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
      
      StatusA     :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
      StatusB     :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
      
      DAC_1_2x    :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
      DAC_3_4x    :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
      DAC_5_6x    :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
      DAC_7_8x    :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0)
   );

END OutputValidStatuses;

ARCHITECTURE RTL OF OutputValidStatuses IS 

SIGNAL enable_delay : STD_LOGIC;
SIGNAL enable_int : STD_LOGIC;

BEGIN 

PROCESS(clk)
 BEGIN
 
   if (rising_edge(clk)) then 
     
      -- use the enable signal for one clock cycle only
      if enableRead = '1' and enable_delay = '0' then
          enable_int <= '1';
      else
          enable_int <= '0';
      end if;
      enable_delay <= enableRead;
      
      -- read the data
      if enable_int = '1' then
         CID <= CID_ChA;
         FID <= FID_ChA;
           
         DAC_1_2x <= DAC_1_2_ChA;
         DAC_3_4x <= DAC_3_4_ChA;
         DAC_5_6x <= DAC_5_6_ChA;
         DAC_7_8x <= DAC_7_8_ChA;
          
         StatusA <= StatusA_ChA;
         StatusB <= StatusB_ChA;
           
         if validPacket = '1' then 
            CID      <= CID_ChB;
            FID      <= FID_ChB;
           
            DAC_1_2x <= DAC_1_2_ChB;
            DAC_3_4x <= DAC_3_4_ChB;
            DAC_5_6x <= DAC_5_6_ChB;
            DAC_7_8x <= DAC_7_8_ChB;
            
            StatusA  <= StatusA_ChB;
            StatusB  <= StatusB_ChB;
         end if;
      end if;
   end if;
  
 END PROCESS;

END RTL;